#!/usr/bin/env node

const { program } = require('commander');
const { authenticate } = require('../lib/auth');
const { compressProject } = require('../lib/compress');
const chalk = require('chalk');
const fs = require('fs');
const path = require('path');

const packageJson = JSON.parse(fs.readFileSync(path.join(__dirname, '../package.json'), 'utf8'));
const version = packageJson.version;

program
  .name('c2ai')
  .description('Code2.ai CLI - Compress and optimize your codebase for AI assistants')
  .version(version);

program
  .command('login')
  .description('Login to Code2.ai')
  .action(async () => {
    try {
      await authenticate(true); // Force login
      console.log(chalk.green('✅ Successfully logged in to Code2.ai'));
    } catch (error) {
      console.error(chalk.red('❌ Login failed:'), error.message);
      process.exit(1);
    }
  });

program
  .command('compress')
  .description('Compress current project for AI assistants')
  .option('-o, --output <path>', 'Output path for the compressed file', './c2ai-context.txt')
  .option('-i, --ignore <patterns>', 'Additional patterns to ignore (comma separated)')
  .option('-k, --keep-all', 'Keep all extracted files in the compressions directory', false)
  .action(async (options) => {
    try {
      // Ensure user is authenticated
      const authToken = await authenticate();
      
      // Process current directory
      await compressProject(process.cwd(), options.output, options.ignore, authToken);
      
      console.log(chalk.blue('📝 Next steps:'));
      console.log('   1. Upload the context file to Claude or ChatGPT');
      console.log('   2. Ask questions about your codebase');
      console.log('   3. You can find additional extracted files in the compressions directory');
    } catch (error) {
      console.error(chalk.red('❌ Compression failed:'), error.message);
      process.exit(1);
    }
  });

// Default command (no args) runs compression
program
  .action(async () => {
    try {
      // Ensure user is authenticated
      const authToken = await authenticate();
      
      // Process current directory with defaults
      await compressProject(process.cwd(), './c2ai-context.txt', null, authToken);
      
      console.log(chalk.blue('📝 Next steps:'));
      console.log('   1. Upload the context file to Claude or ChatGPT');
      console.log('   2. Ask questions about your codebase');
      console.log('   3. You can find additional extracted files in the compressions directory');
    } catch (error) {
      console.error(chalk.red('❌ Compression failed:'), error.message);
      process.exit(1);
    }
  });

program.parse(process.argv);