const Conf = require('conf');
const inquirer = require('inquirer');
const axios = require('axios');
const chalk = require('chalk');
const http = require('http');
const open = require('open');
const url = require('url');
const crypto = require('crypto');

// Create config store for persistent data
const config = new Conf({
  projectName: 'c2ai',
  schema: {
    auth: {
      type: 'object',
      properties: {
        token: { type: 'string' },
        email: { type: 'string' },
        tier: { type: 'string' },
        expiresAt: { type: 'string' }
      }
    }
  }
});

const API_URL = 'http://localhost:5001';

/**
 * Get authentication token, prompting for login if needed
 * @param {boolean} force Force new login even if existing token
 * @returns {Promise<string>} Authentication token
 */
/**
 * Create a local server to receive the OAuth callback
 * @returns {Promise<{server: http.Server, authParams: Promise<Object>}>}
 */
function createLocalServer() {
  // Create the Promise that will be resolved with authentication data
  let resolvePromise, rejectPromise;
  const authPromise = new Promise((resolve, reject) => {
    resolvePromise = resolve;
    rejectPromise = reject;
  });
  
  // Create a server
  const server = http.createServer();
  
  // Add request handler
  server.on('request', (req, res) => {
    const parsedUrl = url.parse(req.url, true);
    const { token, email, tier } = parsedUrl.query;
    
    // Success page
    res.writeHead(200, { 'Content-Type': 'text/html' });
    res.end(`
      <!DOCTYPE html>
      <html>
        <head>
          <title>Authentication Successful</title>
          <style>
            body { 
              font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif; 
              line-height: 1.6;
              max-width: 500px;
              margin: 0 auto;
              padding: 20px;
              text-align: center;
            }
            h1 { color: #4CAF50; }
            .logo { margin: 20px 0; }
            .container {
              border-radius: 8px;
              box-shadow: 0 4px 12px rgba(0,0,0,0.1);
              padding: 30px;
              margin-top: 30px;
            }
          </style>
        </head>
        <body>
          <div class="container">
            <div class="logo">
              <svg width="50" height="50" viewBox="0 0 24 24" fill="#4CAF50">
                <path d="M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm-2 15l-5-5 1.41-1.41L10 14.17l7.59-7.59L19 8l-9 9z"/>
              </svg>
            </div>
            <h1>Successfully Authenticated!</h1>
            <p>You can now close this window and return to the CLI.</p>
          </div>
        </body>
      </html>
    `);
    
    // Check if we have the token and email
    if (token && email) {
      console.log(`Received token for ${email}`);
      resolvePromise({ token, email, tier });
    } else {
      console.log('Invalid authentication response');
      rejectPromise(new Error('Invalid authentication response'));
    }
  });
  
  // Start the server on a random port
  server.listen(0);
  
  // Set a timeout in case we don't get a callback
  const timer = setTimeout(() => {
    rejectPromise(new Error('Authentication timed out after 5 minutes'));
  }, 5 * 60 * 1000);
  
  // Clear the timeout when the promise resolves or rejects
  authPromise.finally(() => clearTimeout(timer));
  
  // Return the server and the promise
  return { server, authParams: authPromise };
}

async function authenticate(force = false) {
  // Check if we have valid auth already
  const auth = config.get('auth');
  
  if (auth?.token && !force && auth.expiresAt && new Date(auth.expiresAt) > new Date()) {
    return auth.token;
  }

  // Need to log in/authenticate
  console.log(chalk.blue('Authentication required for Code2.ai'));

  try {
    // Create a unique state parameter to prevent CSRF
    const state = crypto.randomBytes(16).toString('hex');
    
    // Create a local server to handle the redirect
    const { server, authParams } = createLocalServer();
    const port = server.address().port;
    
    // Construct the authorization URL
    const authUrl = `${API_URL}/cli-auth?redirect_port=${port}&state=${state}`;
    
    console.log(chalk.blue('Opening browser for authentication...'));
    console.log(chalk.yellow('If the browser doesn\'t open automatically, please visit:'));
    console.log(authUrl);
    
    // Open the browser to the authorization URL
    console.log(chalk.blue(`Opening ${authUrl} in your browser...`));
    try {
      await open(authUrl);
    } catch (openError) {
      console.log(chalk.yellow('Could not open browser automatically. Please visit the URL manually.'));
    }
    
    console.log(chalk.blue('Waiting for authentication to complete...'));
    
    // Wait for the callback to be processed
    const { token, email, tier } = await authParams;
    
    // Close the server
    server.close();
    
    console.log(chalk.green('✅ Successfully authenticated!'));
    
    // Store authentication
    config.set('auth', {
      token,
      email,
      tier: tier || 'extension',
      expiresAt: new Date(Date.now() + 7 * 24 * 60 * 60 * 1000).toISOString() // 7 days expiry
    });
    
    return token;
  } catch (error) {
    console.error(chalk.red('Authentication failed:'), error.message);
    throw new Error('Authentication failed');
  }
}

module.exports = { authenticate };